package grafos.dijkstra.modelo;

import grafos.matrizadyacencia.modelo.GrafoMatriz;

// Algoritmo de Dijkstra y operaciones auxiliares
public class CaminoMinimo {
	// Grafo representada mediante una matriz de adyacencia.
	private GrafoMatriz g;
	// Matriz de pesos
	private int[][] pesos;
	// Ultimo vertice en el camino
	private int[] ultimo;
	// Coste minimo. D[]
	private int[] D;
	// Vertices marcados: F[]
	private boolean[] F;
	// vertice origen: s
	private int s;
	// Numero de vertices
	private int n;

	/**
	 * Constructor
	 * 
	 * @param g
	 *            grafo
	 * @param origen
	 *            nodo origen
	 */
	public CaminoMinimo(GrafoMatriz g, String origen) {
		this.g = g;
		n = g.numeroDeVertices();
		this.s = g.numVertice(origen);
		pesos = g.getMatriz();
		ultimo = new int[n];
		D = new int[n];
		F = new boolean[n];
	}

	/**
	 * Metodo que aplica el algorimto de Dijkstra
	 */
	public void caminoMinimos() {
		// Inicializar valores
		for (int i = 0; i < n; i++) {
			F[i] = false;
			D[i] = pesos[s][i];
			ultimo[i] = s;
		}
		// Se marca el nodo origen
		F[s] = true;
		D[s] = 0;
		// Pasos para marcar los n-1 vertices
		for (int i = 1; i < n; i++) {
			// Seleccionar vertice no marcado de menor distancia
			int v = minimo();
			F[v] = true;
			// Actualiza distancia de vertices no marcados
			for (int w = 1; w < n; w++)
				if (!F[w])
					if ((D[v] + pesos[v][w] < D[w])) {
						D[w] = D[v] + pesos[v][w];
						ultimo[w] = v;
					}
		}
		int g = 1;
	}

	/**
	 * Devuelve el vertice no marcado de menor distancia
	 * 
	 * @return el vertice no marcado
	 */
	private int minimo() {
		int mx = GrafoMatriz.INFINITO;
		int v = 1;
		for (int j = 1; j < n; j++)
			if (!F[j] && (D[j] <= mx)) {
				mx = D[j];
				v = j;
			}
		return v;
	}

	/**
	 * Reconstruye el camino de costos minimos desde el origen hasta un vector
	 * cualquiera. Invoca al metodo recursivo
	 * 
	 * @param vertice
	 *            vertice
	 */
	public void recuperarCamino(String vertice) {
		int v = g.numVertice(vertice);
		recuperarCamino(v);
	}

	/**
	 * Reconstruye recursivamete el camino minimo.
	 * 
	 * @param v
	 *            vertice
	 */
	private void recuperarCamino(int v) {
		int anterior = ultimo[v];
		if (v != s) {
			// vuelve al ultimo del ultimo
			recuperarCamino(anterior);
			System.out.print(" -> " + g.nombreVertice(v));
		} else
			System.out.print(" -> " + g.nombreVertice(s));
	}

	public void imprimirCostosMinimos() {
		System.out.println("\nVector de costos minimos:");
		for (int i = 0; i < n; i++)
			System.out.print(D[i] + " ");
	}
}
