package utilidades;
/**
 * Esta clase se utiliza por los grafos
 * 
 * @author libardo
 *
 * @param <T>
 */

public class Cola<T> {
	/**
	 * Primer elemento de la cola
	 */
	private Nodo<T> cabeza;
	/**
	 * Ultimo elemento de la cola
	 */
	private Nodo<T> cola;
	/**
	 * Total de elementos de la cola
	 */
	private int tamanio;

	public Cola() {
		cabeza = null;
		cola = null;
		tamanio = 0;
	}

	public T getCola() {
		return cola.getValor();
	}

	public int getTamanio() {
		return tamanio;
	}

	/**
	 * Verifica si la cola esta vacia
	 * 
	 * @return true si está vacía, false en caso contrario
	 */
	public boolean esVacia() {
		if (cabeza == null)
			return true;
		else
			return false;
	}

	/**
	 * Encolar un elemento nuevo
	 * 
	 * @param valor
	 *            valor a encolar
	 */
	public void encolar(T valor) {
		// Crear un nuevo Nodo
		Nodo<T> nuevo = new Nodo<T>();
		// Fijar el valor dentro del nodo
		nuevo.setValor(valor);
		if (esVacia()) {
			// cabeza y cola apuntan al nodo nuevo
			cabeza = nuevo;
			cola = nuevo;
		} else {
			// Se enlaza el campo siguiente de cola con el nuevo elemento
			cola.setSiguiente(nuevo);
			// La nueva cola pasa a ser nuevo
			cola = nuevo;
		}
		// Incrementa el tamanio porque hay un nuevo elemento en la cola
		tamanio++;
	}

	/**
	 * Elimina el primer elemento de la cola
	 */
	public void desencolar() {
		// si no es vacia
		if (!esVacia()) {
			// verificar si hay un solo elemento en la cola
			if (cabeza == cola) {
				cabeza = null;
				cola = null;
			} else {
				// se elimina el primer elemento de la cola, desplazando la
				// cabeza al siguiente nodo
				cabeza = cabeza.getSiguiente();
			}
			// Se disminuye el total de elementos
			tamanio--;
		}
	}

	/**
	 * Devuelve el primer elemento de la cola
	 * 
	 * @return el primer elemento de la cola
	 */
	public T frente() {
		if (!esVacia())
			return cabeza.getValor();
		else
			return null;
	}

}